@@ -1,4 +1,4 @@
-# Copyright 2019-2020 Siemens AG
+# Copyright 2019-2021 Siemens AG
 # SPDX-License-Identifier: MIT
 
 import logging
@@ -135,7 +135,7 @@ class Folders:
             logger.info(f"{folder} has been updated")
             return folder
         else:
-            description = f"Unable to update folder {folder}"
+            description = f"Unable to update folder {folder.id}"
             raise FossologyApiError(description, response)
 
     def delete_folder(self, folder):
